package com.sun.org.apache.xerces.internal.impl.xs;

import com.sun.org.apache.xerces.internal.xs.XSValue;
import com.sun.org.apache.xerces.internal.xs.ShortList;
import com.sun.org.apache.xerces.internal.xs.XSSimpleTypeDefinition;
import com.sun.org.apache.xerces.internal.impl.xs.util.StringListImpl;
import com.sun.org.apache.xerces.internal.xs.ItemPSVI;
import com.sun.org.apache.xerces.internal.xs.StringList;
import com.sun.org.apache.xerces.internal.impl.dv.ValidatedInfo;
import com.sun.org.apache.xerces.internal.xs.XSTypeDefinition;
import com.sun.org.apache.xerces.internal.xs.XSAttributeDeclaration;
import com.sun.org.apache.xerces.internal.xs.AttributePSVI;

public class AttributePSVImpl implements AttributePSVI
{
    protected XSAttributeDeclaration fDeclaration;
    protected XSTypeDefinition fTypeDecl;
    protected boolean fSpecified;
    protected ValidatedInfo fValue;
    protected short fValidationAttempted;
    protected short fValidity;
    protected String[] fErrors;
    protected String fValidationContext;
    protected boolean fIsConstant;
    
    public AttributePSVImpl() {
        this.fDeclaration = null;
        this.fTypeDecl = null;
        this.fSpecified = false;
        this.fValue = new ValidatedInfo();
        this.fValidationAttempted = 0;
        this.fValidity = 0;
        this.fErrors = null;
        this.fValidationContext = null;
    }
    
    public AttributePSVImpl(boolean isConstant, AttributePSVI attrPSVI) {
        this.fDeclaration = null;
        this.fTypeDecl = null;
        this.fSpecified = false;
        this.fValue = new ValidatedInfo();
        this.fValidationAttempted = 0;
        this.fValidity = 0;
        this.fErrors = null;
        this.fValidationContext = null;
        this.fDeclaration = attrPSVI.getAttributeDeclaration();
        this.fTypeDecl = attrPSVI.getTypeDefinition();
        this.fSpecified = attrPSVI.getIsSchemaSpecified();
        this.fValue.copyFrom(attrPSVI.getSchemaValue());
        this.fValidationAttempted = attrPSVI.getValidationAttempted();
        this.fValidity = attrPSVI.getValidity();
        if ((attrPSVI instanceof AttributePSVImpl)) {
            AttributePSVImpl attrPSVIImpl = (AttributePSVImpl)attrPSVI;
            this.fErrors = ((attrPSVIImpl.fErrors != null) ? ((String[])(String[])attrPSVIImpl.fErrors.clone()) : null);
        }
        else {
            StringList errorCodes = attrPSVI.getErrorCodes();
            int length = errorCodes.getLength();
            if (length > 0) {
                StringList errorMessages = attrPSVI.getErrorMessages();
                String[] errors = new String[length << 1];
                int i = 0;
                int j = 0;
                while (i < length) {
                    errors[j++] = errorCodes.item(i);
                    errors[j++] = errorMessages.item(i);
                    ++i;
                }
                this.fErrors = errors;
            }
        }
        this.fValidationContext = attrPSVI.getValidationContext();
        this.fIsConstant = isConstant;
    }
    
    @Override
    public ItemPSVI constant() {
        if (this.isConstant()) {
            return this;
        }
        return new AttributePSVImpl(true, this);
    }
    
    @Override
    public boolean isConstant() {
        return this.fIsConstant;
    }
    
    @Override
    public String getSchemaDefault() {
        return (this.fDeclaration == null) ? null : this.fDeclaration.getConstraintValue();
    }
    
    @Deprecated
    @Override
    public String getSchemaNormalizedValue() {
        return this.fValue.getNormalizedValue();
    }
    
    @Override
    public boolean getIsSchemaSpecified() {
        return this.fSpecified;
    }
    
    @Override
    public short getValidationAttempted() {
        return this.fValidationAttempted;
    }
    
    @Override
    public short getValidity() {
        return this.fValidity;
    }
    
    @Override
    public StringList getErrorCodes() {
        if (this.fErrors == null || this.fErrors.length == 0) {
            return StringListImpl.EMPTY_LIST;
        }
        return new PSVIErrorList(this.fErrors, true);
    }
    
    @Override
    public StringList getErrorMessages() {
        if (this.fErrors == null || this.fErrors.length == 0) {
            return StringListImpl.EMPTY_LIST;
        }
        return new PSVIErrorList(this.fErrors, false);
    }
    
    @Override
    public String getValidationContext() {
        return this.fValidationContext;
    }
    
    @Override
    public XSTypeDefinition getTypeDefinition() {
        return this.fTypeDecl;
    }
    
    @Override
    public XSSimpleTypeDefinition getMemberTypeDefinition() {
        return this.fValue.getMemberTypeDefinition();
    }
    
    @Override
    public XSAttributeDeclaration getAttributeDeclaration() {
        return this.fDeclaration;
    }
    
    @Deprecated
    @Override
    public Object getActualNormalizedValue() {
        return this.fValue.getActualValue();
    }
    
    @Deprecated
    @Override
    public short getActualNormalizedValueType() {
        return this.fValue.getActualValueType();
    }
    
    @Deprecated
    @Override
    public ShortList getItemValueTypes() {
        return this.fValue.getListValueTypes();
    }
    
    @Override
    public XSValue getSchemaValue() {
        return this.fValue;
    }
    
    public void reset() {
        this.fValue.reset();
        this.fDeclaration = null;
        this.fTypeDecl = null;
        this.fSpecified = false;
        this.fValidationAttempted = 0;
        this.fValidity = 0;
        this.fErrors = null;
        this.fValidationContext = null;
    }
}
